@@ -23,10 +23,18 @@ $strings = array(
 		=> '确认要清除吗?',
 		'Compiling'
 		=> '编译中',
-		'%'
-		=> '%',
+		'% Free'
+		=> '% 剩余',
+		'% Used'
+		=> '% 已用',
 		'Hits'
 		=> '命中',
+		'Hits 24H'
+		=> '24H 分布',
+		'Hits/H'
+		=> '命中/H',
+		'Hits/S'
+		=> '命中/S',
 		'Misses'
 		=> '错过',
 		'Clogs'
@@ -50,6 +58,8 @@ $strings = array(
 		=> '项目',
 		'Refcount'
 		=> '引用数',
+		'PhpShared'
+		=> '共享',
 		'SrcSize'
 		=> '源大小',
 		'Modify'
@@ -58,6 +68,8 @@ $strings = array(
 		=> '设备',
 		'inode'
 		=> 'Inode',
+		'hash'
+		=> 'Hash',
 		'Access'
 		=> '访问',
 		'Create'
@@ -66,6 +78,14 @@ $strings = array(
 		=> '建议参考',
 		'GC'
 		=> 'GC',
+		'Legends:'
+		=> '图例:',
+		'Used Blocks'
+		=> '已用块',
+		'Free Blocks'
+		=> '未用块',
+		'Total'
+		=> '总共',
 		'Cache'
 		=> '缓冲区',
 		'Caches'
