@@ -18,10 +18,14 @@
 #	define ZEND_VM_SET_OPCODE_HANDLER(opline) do { } while (0)
 #endif
 
-#define OP_ZVAL_DTOR(op) do { \
-	Z_UNSET_ISREF(Z_OP_CONSTANT(op)); \
-	zval_dtor(&Z_OP_CONSTANT(op)); \
-} while(0)
+#ifdef ZEND_ENGINE_2_4
+#	define OP_ZVAL_DTOR(op) do { } while(0)
+#else
+#	define OP_ZVAL_DTOR(op) do { \
+		Z_UNSET_ISREF(Z_OP_CONSTANT(op)); \
+		zval_dtor(&Z_OP_CONSTANT(op)); \
+	} while(0)
+#endif
 xc_compile_result_t *xc_compile_result_init(xc_compile_result_t *cr, /* {{{ */
 		zend_op_array *op_array,
 		HashTable *function_table,
@@ -129,12 +133,15 @@ int xc_apply_op_array(xc_compile_result_t *cr, apply_func_t applyer TSRMLS_DC) /
 	return applyer(cr->op_array TSRMLS_CC);
 }
 /* }}} */
-
 int xc_undo_pass_two(zend_op_array *op_array TSRMLS_DC) /* {{{ */
 {
 	zend_op *opline, *end;
 
-#ifndef ZEND_ENGINE_2_4
+#ifdef ZEND_ENGINE_2_4
+	if (!(op_array->fn_flags & ZEND_ACC_DONE_PASS_TWO)) {
+		return 0;
+	}
+#else
 	if (!op_array->done_pass_two) {
 		return 0;
 	}
@@ -143,13 +150,22 @@ int xc_undo_pass_two(zend_op_array *op_array TSRMLS_DC) /* {{{ */
 	opline = op_array->opcodes;
 	end = opline + op_array->last;
 	while (opline < end) {
+#ifdef ZEND_ENGINE_2_4
+		if (opline->op1_type == IS_CONST) {
+			opline->op1.constant = opline->op1.literal - op_array->literals;
+		}
+		if (opline->op2_type == IS_CONST) {
+			opline->op2.constant = opline->op2.literal - op_array->literals;
+		}
+#endif
+
 #ifdef ZEND_ENGINE_2_1
 		switch (opline->opcode) {
 #ifdef ZEND_GOTO
 			case ZEND_GOTO:
 #endif
 			case ZEND_JMP:
-				assert(Z_OP(opline->op1).jmp_addr - op_array->opcodes < op_array->last);
+				assert(Z_OP(opline->op1).jmp_addr >= op_array->opcodes && Z_OP(opline->op1).jmp_addr - op_array->opcodes < op_array->last);
 				Z_OP(opline->op1).opline_num = Z_OP(opline->op1).jmp_addr - op_array->opcodes;
 				break;
 			case ZEND_JMPZ:
@@ -159,14 +175,19 @@ int xc_undo_pass_two(zend_op_array *op_array TSRMLS_DC) /* {{{ */
 #ifdef ZEND_JMP_SET
 			case ZEND_JMP_SET:
 #endif
-				assert(Z_OP(opline->op2).jmp_addr - op_array->opcodes < op_array->last);
+#ifdef ZEND_JMP_SET_VAR
+			case ZEND_JMP_SET_VAR:
+#endif
+				assert(Z_OP(opline->op2).jmp_addr >= op_array->opcodes && Z_OP(opline->op2).jmp_addr - op_array->opcodes < op_array->last);
 				Z_OP(opline->op2).opline_num = Z_OP(opline->op2).jmp_addr - op_array->opcodes;
 				break;
 		}
 #endif
 		opline++;
 	}
-#ifndef ZEND_ENGINE_2_4
+#ifdef ZEND_ENGINE_2_4
+	op_array->fn_flags &= ~ZEND_ACC_DONE_PASS_TWO;
+#else
 	op_array->done_pass_two = 0;
 #endif
 
@@ -176,8 +197,15 @@ int xc_undo_pass_two(zend_op_array *op_array TSRMLS_DC) /* {{{ */
 int xc_redo_pass_two(zend_op_array *op_array TSRMLS_DC) /* {{{ */
 {
 	zend_op *opline, *end;
+#ifdef ZEND_ENGINE_2_4
+	zend_literal *literal = op_array->literals;
+#endif
 
-#ifndef ZEND_ENGINE_2_4
+#ifdef ZEND_ENGINE_2_4
+	if ((op_array->fn_flags & ZEND_ACC_DONE_PASS_TWO)) {
+		return 0;
+	}
+#else
 	if (op_array->done_pass_two) {
 		return 0;
 	}
@@ -187,19 +215,30 @@ int xc_redo_pass_two(zend_op_array *op_array TSRMLS_DC) /* {{{ */
 	op_array->opcodes = (zend_op *) erealloc(op_array->opcodes, sizeof(zend_op)*op_array->last);
 	op_array->size = op_array->last;
 	*/
+#ifdef ZEND_ENGINE_2_4
+	if (literal) {
+		zend_literal *end = literal + op_array->last_literal;
+		while (literal < end) {
+			Z_SET_ISREF(literal->constant);
+			Z_SET_REFCOUNT(literal->constant, 2); /* Make sure is_ref won't be reset */
+			literal++;
+		}
+	}
+#endif
 
 	opline = op_array->opcodes;
 	end = opline + op_array->last;
 	while (opline < end) {
+#ifndef ZEND_ENGINE_2_4
 		if (Z_OP_TYPE(opline->op1) == IS_CONST) {
 			Z_SET_ISREF(Z_OP_CONSTANT(opline->op1));
 			Z_SET_REFCOUNT(Z_OP_CONSTANT(opline->op1), 2); /* Make sure is_ref won't be reset */
-
 		}
 		if (Z_OP_TYPE(opline->op2) == IS_CONST) {
 			Z_SET_ISREF(Z_OP_CONSTANT(opline->op2));
 			Z_SET_REFCOUNT(Z_OP_CONSTANT(opline->op2), 2);
 		}
+#endif
 #ifdef ZEND_ENGINE_2_1
 		switch (opline->opcode) {
 #ifdef ZEND_GOTO
@@ -225,7 +264,9 @@ int xc_redo_pass_two(zend_op_array *op_array TSRMLS_DC) /* {{{ */
 		opline++;
 	}
 
-#ifndef ZEND_ENGINE_2_4
+#ifdef ZEND_ENGINE_2_4
+	op_array->fn_flags |= ZEND_ACC_DONE_PASS_TWO;
+#else
 	op_array->done_pass_two = 1;
 #endif
 	return 0;
@@ -443,7 +484,7 @@ static int xc_do_early_binding(zend_op_array *op_array, HashTable *class_table,
 #endif
 
 #ifdef HAVE_XCACHE_CONSTANT
-void xc_install_constant(char *filename, zend_constant *constant, zend_uchar type, zstr key, uint len, ulong h TSRMLS_DC) /* {{{ */
+void xc_install_constant(ZEND_24(NOTHING, const) char *filename, zend_constant *constant, zend_uchar type, const24_zstr key, uint len, ulong h TSRMLS_DC) /* {{{ */
 {
 	if (zend_u_hash_add(EG(zend_constants), type, key, len,
 				constant, sizeof(zend_constant),
@@ -463,7 +504,7 @@ void xc_install_constant(char *filename, zend_constant *constant, zend_uchar typ
 }
 /* }}} */
 #endif
-void xc_install_function(char *filename, zend_function *func, zend_uchar type, zstr key, uint len, ulong h TSRMLS_DC) /* {{{ */
+void xc_install_function(ZEND_24(NOTHING, const) char *filename, zend_function *func, zend_uchar type, const24_zstr key, uint len, ulong h TSRMLS_DC) /* {{{ */
 {
 	zend_bool istmpkey;
 
@@ -493,7 +534,7 @@ void xc_install_function(char *filename, zend_function *func, zend_uchar type, z
 	}
 }
 /* }}} */
-ZESW(xc_cest_t *, void) xc_install_class(char *filename, xc_cest_t *cest, int oplineno, zend_uchar type, zstr key, uint len, ulong h TSRMLS_DC) /* {{{ */
+ZESW(xc_cest_t *, void) xc_install_class(ZEND_24(NOTHING, const) char *filename, xc_cest_t *cest, int oplineno, zend_uchar type, const24_zstr key, uint len, ulong h TSRMLS_DC) /* {{{ */
 {
 	zend_bool istmpkey;
 	zend_class_entry *cep = CestToCePtr(*cest);
@@ -537,8 +578,61 @@ ZESW(xc_cest_t *, void) xc_install_class(char *filename, xc_cest_t *cest, int op
 #define TG(x) (sandbox->tmp_##x)
 #define OG(x) (sandbox->orig_##x)
 /* }}} */
+#ifdef XCACHE_ERROR_CACHING
+static void xc_sandbox_error_cb(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args) /* {{{ */
+{
+	xc_compilererror_t *compilererror;
+	xc_sandbox_t *sandbox;
+	TSRMLS_FETCH();
+
+	sandbox = (xc_sandbox_t *) XG(sandbox);
+	assert(sandbox != NULL);
+	switch (type) {
+#ifdef E_STRICT
+	case E_STRICT:
+#endif
+#ifdef E_DEPRECATED
+	case E_DEPRECATED:
+#endif
+		if (sandbox->compilererror_cnt <= sandbox->compilererror_size) {
+			if (sandbox->compilererror_size) {
+				sandbox->compilererror_size += 16;
+				sandbox->compilererrors = erealloc(sandbox->compilererrors, sandbox->compilererror_size * sizeof(sandbox->compilererrors));
+			}
+			else {
+				sandbox->compilererror_size = 16;
+				sandbox->compilererrors = emalloc(sandbox->compilererror_size * sizeof(sandbox->compilererrors));
+			}
+		}
+		compilererror = &sandbox->compilererrors[sandbox->compilererror_cnt++];
+		compilererror->type = type;
+		compilererror->lineno = error_lineno;
+		compilererror->error_len = vspprintf(&compilererror->error, 0, format, args);
+		break;
+
+	default: {
+		/* give up, and user handler is not supported in this case */
+		zend_uint i;
+		zend_uint orig_lineno = CG(zend_lineno);
+		zend_error_cb = sandbox->orig_zend_error_cb;
+
+		for (i = 0; i < sandbox->compilererror_cnt; i ++) {
+			compilererror = &sandbox->compilererrors[i];
+			CG(zend_lineno) = compilererror->lineno;
+			zend_error(compilererror->type, "%s", compilererror->error);
+		}
+		CG(zend_lineno) = orig_lineno;
+		sandbox->compilererror_cnt = 0;
+
+		sandbox->orig_zend_error_cb(type, error_filename, error_lineno, format, args);
+		break;
+	}
+	}
+}
+/* }}} */
+#endif
 #ifdef ZEND_ENGINE_2_1
-static zend_bool xc_auto_global_callback(char *name, uint name_len TSRMLS_DC) /* {{{ */
+static zend_bool xc_auto_global_callback(ZEND_24(NOTHING, const) char *name, uint name_len TSRMLS_DC) /* {{{ */
 {
 	return 0;
 }
@@ -571,7 +665,7 @@ void xc_hash_copy_if(HashTable *target, HashTable *source, copy_ctor_func_t pCop
 				target->pInternalPointer = NULL;
 			}
 			if (p->nKeyLength) {
-				zend_hash_quick_update(target, p->arKey, p->nKeyLength, p->h, p->pData, size, &new_entry);
+				zend_u_hash_quick_update(target, p->key.type, ZSTR(BUCKET_KEY_S(p)), p->nKeyLength, p->h, p->pData, size, &new_entry);
 			} else {
 				zend_hash_index_update(target, p->h, p->pData, size, &new_entry);
 			}
@@ -595,16 +689,12 @@ static zend_bool xc_is_internal_zend_constant(zend_constant *c) /* {{{ */
 void xc_zend_constant_ctor(zend_constant *c) /* {{{ */
 {
 	assert((c->flags & CONST_PERSISTENT));
-#ifdef IS_UNICODE
-	c->name.u = zend_ustrndup(c->name.u, c->name_len - 1);
-#else
-	c->name = zend_strndup(c->name, c->name_len - 1);
-#endif
+	ZSTR_U(c->name) = UNISW(zend_strndup, zend_ustrndup)(ZSTR_U(c->name), c->name_len - 1);
 }
 /* }}} */
 void xc_zend_constant_dtor(zend_constant *c) /* {{{ */
 {
-	free(ZSTR_U(c->name));
+	free(ZSTR_V(c->name));
 }
 /* }}} */
 static void xc_free_zend_constant(zend_constant *c) /* {{{ */
@@ -622,7 +712,7 @@ void xc_copy_internal_zend_constants(HashTable *target, HashTable *source) /* {{
 }
 /* }}} */
 #endif
-xc_sandbox_t *xc_sandbox_init(xc_sandbox_t *sandbox, char *filename TSRMLS_DC) /* {{{ */
+xc_sandbox_t *xc_sandbox_init(xc_sandbox_t *sandbox, ZEND_24(NOTHING, const) char *filename TSRMLS_DC) /* {{{ */
 {
 	HashTable *h;
 
@@ -663,7 +753,7 @@ xc_sandbox_t *xc_sandbox_init(xc_sandbox_t *sandbox, char *filename TSRMLS_DC) /
 	TG(internal_constant_tail) = TG(zend_constants).pListTail;
 #endif
 	h = OG(function_table);
-	zend_hash_init_ex(&TG(function_table), 128, NULL, h->pDestructor, h->persistent, h->bApplyProtection);
+	zend_hash_init_ex(&TG(function_table), 128, NULL, ZEND_FUNCTION_DTOR, h->persistent, h->bApplyProtection);
 	{
 		zend_function tmp_func;
 		zend_hash_copy(&TG(function_table), &XG(internal_function_table), NULL, (void *) &tmp_func, sizeof(tmp_func));
@@ -671,7 +761,7 @@ xc_sandbox_t *xc_sandbox_init(xc_sandbox_t *sandbox, char *filename TSRMLS_DC) /
 	TG(internal_function_tail) = TG(function_table).pListTail;
 
 	h = OG(class_table);
-	zend_hash_init_ex(&TG(class_table),     16, NULL, h->pDestructor, h->persistent, h->bApplyProtection);
+	zend_hash_init_ex(&TG(class_table),     16, NULL, ZEND_CLASS_DTOR, h->persistent, h->bApplyProtection);
 #if 0 && TODO
 	{
 		xc_cest_t tmp_cest;
@@ -694,9 +784,14 @@ xc_sandbox_t *xc_sandbox_init(xc_sandbox_t *sandbox, char *filename TSRMLS_DC) /
 
 	sandbox->filename = filename;
 
-#ifdef E_STRICT
+#ifdef XCACHE_ERROR_CACHING
 	sandbox->orig_user_error_handler_error_reporting = EG(user_error_handler_error_reporting);
-	EG(user_error_handler_error_reporting) &= ~E_STRICT;
+	EG(user_error_handler_error_reporting) = 0;
+
+	sandbox->compilererror_cnt  = 0;
+	sandbox->compilererror_size = 0;
+	sandbox->orig_zend_error_cb = zend_error_cb;
+	zend_error_cb = xc_sandbox_error_cb;
 #endif
 
 #ifdef ZEND_COMPILE_IGNORE_INTERNAL_CLASSES
@@ -706,6 +801,7 @@ xc_sandbox_t *xc_sandbox_init(xc_sandbox_t *sandbox, char *filename TSRMLS_DC) /
 	CG(compiler_options) |= ZEND_COMPILE_IGNORE_INTERNAL_CLASSES | ZEND_COMPILE_NO_CONSTANT_SUBSTITUTION | ZEND_COMPILE_DELAYED_BINDING;
 #endif
 
+	XG(sandbox) = (void *) sandbox;
 	return sandbox;
 }
 /* }}} */
@@ -776,12 +872,28 @@ static void xc_sandbox_install(xc_sandbox_t *sandbox, xc_install_action_t instal
 #endif
 	}
 
+#ifdef XCACHE_ERROR_CACHING
+	/* restore trigger errors */
+	for (i = 0; i < sandbox->compilererror_cnt; i ++) {
+		xc_compilererror_t *error = &sandbox->compilererrors[i];
+		CG(zend_lineno) = error->lineno;
+		zend_error(error->type, "%s", error->error);
+	}
+	CG(zend_lineno) = 0;
+#endif
+
 	i = 1;
 	zend_hash_add(&OG(included_files), sandbox->filename, strlen(sandbox->filename) + 1, (void *)&i, sizeof(int), NULL);
 }
 /* }}} */
 void xc_sandbox_free(xc_sandbox_t *sandbox, xc_install_action_t install TSRMLS_DC) /* {{{ */
 {
+	XG(sandbox) = NULL;
+#ifdef XCACHE_ERROR_CACHING
+	EG(user_error_handler_error_reporting) = sandbox->orig_user_error_handler_error_reporting;
+	zend_error_cb = sandbox->orig_zend_error_cb;
+#endif
+
 	/* restore first first install function/class */
 #ifdef HAVE_XCACHE_CONSTANT
 	EG(zend_constants) = OG(zend_constants);
@@ -795,7 +907,7 @@ void xc_sandbox_free(xc_sandbox_t *sandbox, xc_install_action_t install TSRMLS_D
 
 	if (install != XC_NoInstall) {
 		CG(in_compilation)    = 1;
-		CG(compiled_filename) = sandbox->filename;
+		CG(compiled_filename) = ZEND_24(NOTHING, (char *)) sandbox->filename;
 		CG(zend_lineno)       = 0;
 		xc_sandbox_install(sandbox, install TSRMLS_CC);
 		CG(in_compilation)    = 0;
@@ -823,8 +935,14 @@ void xc_sandbox_free(xc_sandbox_t *sandbox, xc_install_action_t install TSRMLS_D
 	/* restore orig here, as EG/CG holded tmp before */
 	memcpy(&EG(included_files), &OG(included_files), sizeof(EG(included_files)));
 
-#ifdef E_STRICT
-	EG(user_error_handler_error_reporting) = sandbox->orig_user_error_handler_error_reporting;
+#ifdef XCACHE_ERROR_CACHING
+	if (sandbox->compilererrors) {
+		zend_uint i;
+		for (i = 0; i < sandbox->compilererror_cnt; i ++) {
+			efree(sandbox->compilererrors[i].error);
+		}
+		efree(sandbox->compilererrors);
+	}
 #endif
 
 #ifdef ZEND_COMPILE_IGNORE_INTERNAL_CLASSES
@@ -854,14 +972,19 @@ int xc_trace(const char *fmt, ...) /* {{{ */
 /* }}} */
 
 #ifndef ZEND_ENGINE_2_3
-size_t zend_dirname(char *path, size_t len) /* {{{ */
+#include "ext/standard/php_string.h"
+size_t xc_dirname(char *path, size_t len) /* {{{ */
 {
+#ifdef ZEND_ENGINE_2
+	return php_dirname(path, len);
+#else
 	php_dirname(path, len);
 	return strlen(path);
+#endif
 }
 /* }}} */
 
-long zend_atol(const char *str, int str_len) /* {{{ */
+long xc_atol(const char *str, int str_len) /* {{{ */
 {
 	long retval;
 
