@@ -1,9 +1,9 @@
 #ifndef __XCACHE_H
 #define __XCACHE_H
 #define XCACHE_NAME       "XCache"
-#define XCACHE_VERSION    "1.3.2"
+#define XCACHE_VERSION    "2.0.0"
 #define XCACHE_AUTHOR     "mOo"
-#define XCACHE_COPYRIGHT  "Copyright (c) 2005-2011"
+#define XCACHE_COPYRIGHT  "Copyright (c) 2005-2012"
 #define XCACHE_URL        "http://xcache.lighttpd.net"
 #define XCACHE_WIKI_URL   XCACHE_URL "/wiki"
 
@@ -21,7 +21,7 @@
 #include "lock.h"
 
 #define HAVE_INODE
-#if !defined(ZEND_ENGINE_2_4) && (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION == 3 && PHP_RELEASE_VERSION >= 99 || PHP_MAJOR_VERSION > 5)
+#if !defined(ZEND_ENGINE_2_4) && (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 4 || PHP_MAJOR_VERSION > 5)
 #	define ZEND_ENGINE_2_4
 #endif
 #if !defined(ZEND_ENGINE_2_3) && (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION == 3 || defined(ZEND_ENGINE_2_4))
@@ -41,6 +41,11 @@
 #else
 #	define ZESW(v1, v2) v2
 #endif
+#ifdef ZEND_ENGINE_2_4
+#	define ZEND_24(pre24, v24) v24
+#else
+#	define ZEND_24(pre24, v24) pre24
+#endif
 
 #ifdef do_alloca_with_limit
 #	define my_do_alloca(size, use_heap) do_alloca_with_limit(size, use_heap)
@@ -130,18 +135,33 @@ typedef znode znode_op;
 				? UBYTES(b->nKeyLength) \
 				: b->nKeyLength \
 				))
-#define BUCKET_KEY_S(b)    (UNISW((b)->arKey, (b)->key.arKey.s))
-#define BUCKET_KEY_U(b)    (UNISW((b)->arKey, (b)->key.arKey.u))
+#define BUCKET_KEY(b)      (UNISW((b)->arKey, (b)->key.arKey))
+#define BUCKET_KEY_S(b)    ZSTR_S(BUCKET_KEY(b))
+#define BUCKET_KEY_U(b)    ZSTR_U(BUCKET_KEY(b))
 #define BUCKET_KEY_TYPE(b) (UNISW(IS_STRING,  (b)->key.type))
 #ifdef IS_UNICODE
 #	define BUCKET_HEAD_SIZE(b) XtOffsetOf(Bucket, key.arKey)
 #else
 #	define BUCKET_HEAD_SIZE(b) XtOffsetOf(Bucket, arKey)
 #endif
-#define BUCKET_SIZE(b) (BUCKET_HEAD_SIZE(b) + BUCKET_KEY_SIZE(b))
+
+#ifdef ZEND_ENGINE_2_4
+#	define BUCKET_SIZE(b) (sizeof(Bucket) + BUCKET_KEY_SIZE(b))
+#else
+#	define BUCKET_SIZE(b) (BUCKET_HEAD_SIZE(b) + BUCKET_KEY_SIZE(b))
+#endif
 
 #ifndef IS_UNICODE
 typedef char *zstr;
+typedef const char *const_zstr;
+#ifdef ZEND_ENGINE_2_4
+typedef const char *const24_zstr;
+typedef const char *const24_str;
+#else
+typedef char *const24_zstr;
+typedef char *const24_str;
+#endif
+
 #	define ZSTR_S(s)     (s)
 #	define ZSTR_U(s)     (s)
 #	define ZSTR_V(s)     (s)
@@ -149,6 +169,7 @@ typedef char *zstr;
 #	define ZSTR_PU(s)    (s)
 #	define ZSTR_PV(s)    (s)
 #else
+typedef const zstr const_zstr;
 #	define ZSTR_S(zs)    ((zs).s)
 #	define ZSTR_U(zs)    ((zs).u)
 #	define ZSTR_V(zs)    ((zs).v)
@@ -169,22 +190,25 @@ typedef char *zstr;
 /* {{{ u hash wrapper */
 #ifndef IS_UNICODE
 #	define zend_u_hash_add(ht, type, arKey, nKeyLength, pData, nDataSize, pDest) \
- 	   zend_hash_add(ht, arKey, nKeyLength, pData, nDataSize, pDest)
+ 	   zend_hash_add(ht, ZEND_24((char *), NOTHING) arKey, nKeyLength, pData, nDataSize, pDest)
 
 #	define zend_u_hash_quick_add(ht, type, arKey, nKeyLength, h, pData, nDataSize, pDest) \
- 	   zend_hash_quick_add(ht, arKey, nKeyLength, h, pData, nDataSize, pDest)
+ 	   zend_hash_quick_add(ht, ZEND_24((char *), NOTHING) arKey, nKeyLength, h, pData, nDataSize, pDest)
 
 #	define zend_u_hash_update(ht, type, arKey, nKeyLength, pData, nDataSize, pDest) \
- 	   zend_hash_update(ht, arKey, nKeyLength, pData, nDataSize, pDest)
+ 	   zend_hash_update(ht, ZEND_24((char *), NOTHING) arKey, nKeyLength, pData, nDataSize, pDest)
 
 #	define zend_u_hash_quick_update(ht, type, arKey, nKeyLength, h, pData, nDataSize, pDest) \
- 	   zend_hash_quick_update(ht, arKey, nKeyLength, h, pData, nDataSize, pDest)
+ 	   zend_hash_quick_update(ht, ZEND_24((char *), NOTHING) arKey, nKeyLength, h, pData, nDataSize, pDest)
 
 #	define zend_u_hash_find(ht, type, arKey, nKeyLength, pData) \
- 	   zend_hash_find(ht, arKey, nKeyLength, pData)
+ 	   zend_hash_find(ht, ZEND_24((char *), NOTHING) arKey, nKeyLength, pData)
 
 #	define zend_u_hash_quick_find(ht, type, arKey, nKeyLength, h, pData) \
- 	   zend_hash_quick_find(ht, arKey, nKeyLength, h, pData)
+ 	   zend_hash_quick_find(ht, ZEND_24((char *), NOTHING) arKey, nKeyLength, h, pData)
+
+#	define zend_u_hash_exists(ht, type, arKey, nKeyLength) \
+ 	   zend_hash_exists(ht, ZEND_24((char *), NOTHING) arKey, nKeyLength)
 
 #	define add_u_assoc_zval_ex(arg, type, key, key_len, value) \
 		add_assoc_zval_ex(arg, key, key_len, value)
@@ -226,8 +250,9 @@ typedef ZESW(zend_class_entry, zend_class_entry*) xc_cest_t;
 
 typedef zend_op_array *(zend_compile_file_t)(zend_file_handle *h, int type TSRMLS_DC);
 
-/* {{{ xc_cache_t */
 typedef struct _xc_entry_t xc_entry_t;
+typedef struct _xc_entry_data_php_t xc_entry_data_php_t;
+/* {{{ xc_cache_t */
 typedef struct {
 	int cacheid;
 	xc_hash_t  *hcache; /* hash to cacheid */
@@ -239,14 +264,17 @@ typedef struct {
 	zend_ulong ooms;
 	zend_ulong errors;
 	xc_lock_t  *lck;
-	xc_shm_t   *shm; /* to which shm contains us */
-	xc_mem_t   *mem; /* to which mem contains us */
+	xc_shm_t   *shm; /* which shm contains us */
+	xc_mem_t   *mem; /* which mem contains us */
 
 	xc_entry_t **entries;
 	int entries_count;
+	xc_entry_data_php_t **phps;
+	int phps_count;
 	xc_entry_t *deletes;
 	int deletes_count;
-	xc_hash_t  *hentry; /* hash to entry */
+	xc_hash_t  *hentry; /* hash settings to entry */
+	xc_hash_t  *hphp;   /* hash settings to php */
 
 	time_t     last_gc_deletes;
 	time_t     last_gc_expires;
@@ -259,17 +287,36 @@ typedef struct {
 	zend_ulong hits_by_second[5];
 } xc_cache_t;
 /* }}} */
+/* {{{ xc_op_array_info_detail_t */
+typedef struct {
+	zend_uint index;
+	zend_uint info;
+} xc_op_array_info_detail_t;
+/* }}} */
+/* {{{ xc_op_array_info_t */
+typedef struct {
+#ifdef ZEND_ENGINE_2_4
+	zend_uint literalinfo_cnt;
+	xc_op_array_info_detail_t *literalinfos;
+#else
+	zend_uint oplineinfo_cnt;
+	xc_op_array_info_detail_t *oplineinfos;
+#endif
+} xc_op_array_info_t;
+/* }}} */
 /* {{{ xc_classinfo_t */
 typedef struct {
 #ifdef IS_UNICODE
-	zend_uchar type;
+	zend_uchar   type;
 #endif
-	zstr      key;
-	zend_uint key_size;
-	ulong     h;
-	xc_cest_t cest;
+	const24_zstr key;
+	zend_uint    key_size;
+	ulong        h;
+	zend_uint  methodinfo_cnt;
+	xc_op_array_info_t *methodinfos;
+	xc_cest_t    cest;
 #ifndef ZEND_COMPILE_DELAYED_BINDING
-	int       oplineno;
+	int          oplineno;
 #endif
 } xc_classinfo_t;
 /* }}} */
@@ -277,11 +324,11 @@ typedef struct {
 /* {{{ xc_constinfo_t */
 typedef struct {
 #ifdef IS_UNICODE
-	zend_uchar type;
+	zend_uchar    type;
 #endif
-	zstr      key;
-	zend_uint key_size;
-	ulong     h;
+	const24_zstr  key;
+	zend_uint     key_size;
+	ulong         h;
 	zend_constant constant;
 } xc_constinfo_t;
 /* }}} */
@@ -289,11 +336,12 @@ typedef struct {
 /* {{{ xc_funcinfo_t */
 typedef struct {
 #ifdef IS_UNICODE
-	zend_uchar type;
+	zend_uchar     type;
 #endif
-	zstr      key;
-	zend_uint key_size;
-	ulong     h;
+	const24_zstr   key;
+	zend_uint      key_size;
+	ulong          h;
+	xc_op_array_info_t op_array_info;
 	zend_function func;
 } xc_funcinfo_t;
 /* }}} */
@@ -301,24 +349,37 @@ typedef struct {
 /* {{{ xc_autoglobal_t */
 typedef struct {
 #ifdef IS_UNICODE
-	zend_uchar type;
+	zend_uchar   type;
 #endif
-	zstr       key;
-	zend_uint  key_len;
-	ulong      h;
+	const24_zstr key;
+	zend_uint    key_len;
+	ulong        h;
 } xc_autoglobal_t;
 /* }}} */
 #endif
-typedef enum { XC_TYPE_PHP, XC_TYPE_VAR } xc_entry_type_t;
-/* {{{ xc_entry_data_php_t */
 typedef struct {
-	size_t sourcesize;
-#ifdef HAVE_INODE
-	int device;             /* the filesystem device */
-	int inode;              /* the filesystem inode */
-#endif
-	time_t mtime;           /* the mtime of origin source file */
+	char digest[16];
+} xc_md5sum_t;
+/* {{{ xc_compilererror_t */
+typedef struct {
+	int type;
+	uint lineno;
+	int error_len;
+	char *error;
+} xc_compilererror_t;
+/* }}} */
+/* {{{ xc_entry_data_php_t */
+struct _xc_entry_data_php_t {
+	xc_entry_data_php_t *next;
+	xc_hash_value_t      hvalue;
 
+	xc_md5sum_t md5;        /* md5sum of the source */
+	zend_ulong  refcount;   /* count of entries referencing to this data */
+
+	zend_ulong hits;        /* hits of this php */
+	size_t     size;
+
+	xc_op_array_info_t op_array_info;
 	zend_op_array *op_array;
 
 #ifdef HAVE_XCACHE_CONSTANT
@@ -339,41 +400,78 @@ typedef struct {
 	zend_uint autoglobal_cnt;
 	xc_autoglobal_t *autoglobals;
 #endif
-} xc_entry_data_php_t;
-/* }}} */
-/* {{{ xc_entry_data_var_t */
-typedef struct {
-	zval   *value;
-} xc_entry_data_var_t;
+
+#ifdef E_STRICT
+	zend_uint compilererror_cnt;
+	xc_compilererror_t *compilererrors;
+#endif
+
+	zend_bool  have_references;
+};
 /* }}} */
 typedef zvalue_value xc_entry_name_t;
 /* {{{ xc_entry_t */
 struct _xc_entry_t {
-	xc_entry_type_t type;
-	xc_hash_value_t hvalue;
 	xc_entry_t *next;
-	xc_cache_t *cache;      /* which cache it's on */
 
 	size_t     size;
-	zend_ulong refcount;
+	time_t     ctime;           /* creation ctime of this entry */
+	time_t     atime;           /*   access atime of this entry */
+	time_t     dtime;           /*  deletion time of this entry */
 	zend_ulong hits;
-	time_t     ctime;           /* the ctime of this entry */
-	time_t     atime;           /* the atime of this entry */
-	time_t     dtime;           /* the deletion time of this entry */
-	long       ttl;             /* ttl of time entry, var only */
+	long       ttl;
 
+	xc_entry_name_t name;
+};
+
+typedef struct {
+	xc_entry_t entry;
+	xc_entry_data_php_t *php;
+
+	zend_ulong refcount;    /* count of php instances holding this entry */
+	time_t file_mtime;
+	size_t file_size;
+#ifdef HAVE_INODE
+	int file_device;
+	int file_inode;
+#endif
+
+	int    filepath_len;
+	ZEND_24(NOTHING, const) char *filepath;
+	int    dirpath_len;
+	char  *dirpath;
+#ifdef IS_UNICODE
+	int    ufilepath_len;
+	UChar *ufilepath;
+	int    udirpath_len;
+	UChar *udirpath;
+#endif
+} xc_entry_php_t;
+
+typedef struct {
+	xc_entry_t entry;
 #ifdef IS_UNICODE
 	zend_uchar name_type;
 #endif
-	xc_entry_name_t name;
-
-	union {
-		xc_entry_data_php_t *php;
-		xc_entry_data_var_t *var;
-	} data;
-
+	zval      *value;
 	zend_bool  have_references;
-};
+} xc_entry_var_t;
+/* }}} */
+typedef struct xc_entry_hash_t { /* {{{ */
+	xc_hash_value_t cacheid;
+	xc_hash_value_t entryslotid;
+} xc_entry_hash_t;
+/* }}} */
+typedef struct xc_compiler_t { /* {{{ */
+	const char *filename;
+	size_t filename_len;
+	const char *opened_path;
+	char opened_path_buffer[MAXPATHLEN];
+
+	xc_entry_hash_t entry_hash;
+	xc_entry_php_t new_entry;
+	xc_entry_data_php_t new_php;
+} xc_compiler_t;
 /* }}} */
 
 extern zend_module_entry xcache_module_entry;
@@ -382,6 +480,16 @@ extern zend_module_entry xcache_module_entry;
 int xc_is_rw(const void *p);
 int xc_is_ro(const void *p);
 int xc_is_shm(const void *p);
-void xc_gc_add_op_array(zend_op_array *op_array TSRMLS_DC);
+/* {{{ xc_gc_op_array_t */
+typedef struct {
+#ifdef ZEND_ENGINE_2
+	zend_uint num_args;
+	zend_arg_info *arg_info;
+#endif
+	zend_op *opcodes;
+} xc_gc_op_array_t;
+/* }}} */
+void xc_gc_add_op_array(xc_gc_op_array_t *gc_op_array TSRMLS_DC);
+void xc_fix_op_array_info(const xc_entry_php_t *xce, const xc_entry_data_php_t *php, zend_op_array *op_array, int shallow_copy, const xc_op_array_info_t *op_array_info TSRMLS_DC);
 
 #endif /* __XCACHE_H */
