@@ -3,6 +3,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xcache)
 	zend_bool initial_compile_file_called; /* true is origin_compile_file is called */
 	zend_bool cacher;      /* true if enabled */
 	zend_bool stat;
+	zend_bool experimental;
 #ifdef HAVE_XCACHE_OPTIMIZER
 	zend_bool optimizer;   /* true if enabled */
 #endif
@@ -17,9 +18,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xcache)
 	long   var_ttl;
 	zend_bool auth_enabled;
 
-#ifdef ZEND_ENGINE_2
-	HashTable gc_op_arrays;
-#endif
+	zend_llist gc_op_arrays;
 
 #ifdef HAVE_XCACHE_CONSTANT
 	HashTable internal_constant_table;
@@ -27,6 +26,8 @@ ZEND_BEGIN_MODULE_GLOBALS(xcache)
 	HashTable internal_function_table;
 	HashTable internal_class_table;
 	zend_bool internal_table_copied;
+
+	void *sandbox;
 ZEND_END_MODULE_GLOBALS(xcache)
 
 ZEND_EXTERN_MODULE_GLOBALS(xcache)
